
angular.module('OBApp.finance', [

])

.config(function($stateProvider, $urlRouterProvider) {

  $stateProvider
  .state('tab.finance', {
      url: '/finance?key&category',
      cache:false,
      views: {
          'tab-news': {
              templateUrl: 'finance/index.html',
              controller: 'FinanceListController'
          }
      }
  })
  .state('tab.financeCategory', {
      url: '/financeCategory',
      cache:false,
      views: {
          'tab-news': {
              templateUrl: 'finance/category.html',
              controller: 'FinanceCategoryController'
          }
      }
  })

  .state('tab.financeDetail', {
      url: '/finance/financeDetail/{id}',
      cache:false,
      views: {
          'tab-news': {
              templateUrl: 'finance/detail.html',
              controller: 'FinanceDetailController'
          }
      },
      resolve: {
          OpenCloseParams: function (TradeService) {
              return TradeService.getOpenAndCloseData().then(function (data) {
                  return data;
              })
          }
          ,
          LimitPriceUpAndDown:function ($stateParams,TradeService) {
              return TradeService.queryLimitPriceUpAndDown($stateParams.id).then(function (data) {
                  return data;
              })
          }
      }
  })

  .state('tab.financeBuy', {
      url: '/finance/buy/{id:int}?:from',
      cache:false,

      views: {
          'tab-news': {
              templateUrl: 'trade/buy.html',
              controller: 'TradeBuyController'
          }
      },
      resolve:{
          userCapatial: function (TradeService) {
              return TradeService.getUserCapatial().then(function(data){
                  return data;
              });
          },
          userData: function (TradeService,SignService) {
              return TradeService.queryUserContactInfo(SignService.getUserId()).then(function(data){
                  return data.data;
              });
          }
      }
  })
})


.controller('FinanceCategoryController', function ($scope, $stateParams,ProductService) {
    $scope.data = [];
    ProductService.getProductCategory().then(function (data) {
        $scope.data = data.configs;
    })
})
    .controller('FinanceListController', function ($scope, $stateParams,ProductService, TradeService) {
        if($stateParams.category){
            ProductService.getProductListWithCategory($stateParams.category).then(function(data){
                $scope.productList = data;

                TradeService.queryCurrentDayData().then(function(data){
                    angular.forEach($scope.productList, function(product, key) {
                        angular.forEach(data, function(tradeData, key) {
                            if(product.id == tradeData.teaId){
                                product.totalAmount = tradeData.totalAmount;
                                // product.tradeData = tradeData;
                            }
                        });
                    });
                });
            });
        }else{
            ProductService.getProductList().then(function(data){
                $scope.productList = data;

                TradeService.queryCurrentDayData().then(function(data){
                    angular.forEach($scope.productList, function(product, key) {
                        angular.forEach(data, function(tradeData, key) {
                            if(product.id == tradeData.teaId){
                                product.totalAmount = tradeData.totalAmount;
                                // product.tradeData = tradeData;
                            }
                        });
                    });
                });
            });
        }

        $scope.trendMode = 'percent';
        $scope.orderBy = $stateParams.key || '-name';
        $scope.activeSegement = { key : $scope.orderBy };


        $scope.segements = [//key是排序字段
            { title: '茶叶指数', key:'-name'},
            { title: '涨跌排行', key:'-trendingPercent' },
            { title: '交易排行', key:'-totalAmount' }
        ];

        $scope.segementChanged = function(selectedSeg){
            $scope.orderBy = selectedSeg.key;
        }

        $scope.showTrendMode = function(mode){//trendMode有两个，一是price,二是百分比
            $scope.trendMode = mode;
        }

        $scope.trendTitle = function(){
            if($scope.orderBy === '-totalAmount'){
                return "成交量";
            }else{
                return $scope.trendMode === 'price' ? "涨幅(价格)" : "涨/跌(百分比)";
            }
        }

    })

    .controller('FinanceDetailController', function ($scope, $filter, $stateParams, $location, ProductService, TradeService,FavouriteService,SignService,$ionicActionSheet,OpenCloseParams,LimitPriceUpAndDown) {
        $scope.LimitPriceUpAndDown = LimitPriceUpAndDown;
        $scope.f = {};

        $scope.chartConfig = {
            options: {
                legend:{
                    enabled:false
                },
                plotOptions:{
                    spline:{
                        allowPointSelect:false
                    },
                    series: {
                        marker: {
                            enabled: false
                        }
                    }
                },
                tooltip: {
                    style: {
                        padding: 10,
                        fontWeight: 'bold'
                    }
                },
            },
            title: {
                text: ''
            },

            series: [],
            xAxis: {
            },
            yAxis:{
                title:''
            },
            size:{
                height:200
            }
        };
        $scope.productSegements = [
            { title: '行情', key:'market'},
            { title: '产品详情', key:'detail'}
        ];
        $scope.activeProductSegement = $scope.productSegements[0];
        $scope.userLogined = function(){
            return SignService.userLogined();
        }
        $scope.productSegementChanged = function (selectedSeg) {
            $scope.activeProductSegement = selectedSeg;
        }
        $scope.modal = null;
        if(SignService.userLogined()) {
            FavouriteService.isMyFavourite(SignService.getUserId(), $stateParams.id).then(function (data) {
                $scope.isMyFavourite = data.isFavourite;
            })
        }
        $scope.f.id = $stateParams.id;

        $scope.loadChartData = function (params) {
            if(angular.isUndefined(params.interval)){

                var date = new Date();
                var minValue = Date.UTC(date.getFullYear(),date.getMonth(),date.getDate(),parseInt(OpenCloseParams.start),0,0);
                $scope.chartConfig.yAxis = {};
                $scope.chartConfig.yAxis.title = "";
                $scope.chartConfig.yAxis.min = $scope.LimitPriceUpAndDown.down;
                $scope.chartConfig.yAxis.max = $scope.LimitPriceUpAndDown.up;

                $scope.chartConfig.yAxis.plotLines = [{
                    color:'#DAA520',           //线的颜色
                    dashStyle:'shortdash',     //虚线
                    value:$scope.productDetail.priceLastDayFinish,               //昨日收盘价
                    width:2                //线的宽度
                }];
                $scope.chartConfig.xAxis = {};
                $scope.chartConfig.xAxis = {
                    type: 'datetime',
                    min: minValue,
                    startOnTick: true,
                    tickInterval:600000,
                    dateTimeLabelFormats :  {
                        second: '%Y-%m-%d<br/>%H:%M:%S'
                    }
                }
            }else{
                $scope.chartConfig.xAxis = {};
                $scope.chartConfig.yAxis = {};
                $scope.chartConfig.yAxis.title = "";
                $scope.chartConfig.yAxis.min =null;
                $scope.chartConfig.yAxis.max = null;

                $scope.chartConfig.yAxis.plotLines = [];
                $scope.chartConfig.xAxis = {
                    type: 'datetime',
                    min:null,
                    tickInterval:null,
                    startOnTick:false,
                    dateTimeLabelFormats:{
                        day: '%m-%d',
                        week: '%m-%d',
                        month: '%Y-%m',
                        year: '%Y'
                    }
                }
            }

            TradeService.queryChartData(params).then(function(chart){
                $scope.chartConfig.series = [];
                var tmp  = {id:0,type: "spline",data:chart.data,name:$scope.productDetail.name};
                $scope.chartConfig.series.push(tmp);
            });
        }


        $scope.id = $stateParams.id;
        ProductService.getProductDetail($scope.id).then(function(productDetail){
            $scope.productDetail = productDetail;
            $scope.loadChartData($scope.f);
            TradeService.queryCurrentDayData().then(function(data){
                angular.forEach(data, function(tradeData, key) {
                    if($scope.productDetail.id == tradeData.teaId){
                        $scope.productDetail.tradeData = tradeData;
                    }
                });
            });
        });

        $scope.productActions = [
            { title: '查看商品简介', url:'viewDesc' },
            { title: '查看我的交易明细', url:'viewTrades' }
        ];

        $scope.buy = function(){
            $location.path("/tab/product/buy/"+$scope.id);
        };

        $scope.shareViaWechat = function (message,title,desc,mediaTag,type,thumb,fnSuccess,fnFailure) {
            Wechat.share({
                message: {
                    title: title,
                    description: desc,
                    mediaTagName: mediaTag,
                    thumb: thumb,
                    media: {
                        type: Wechat.Type.WEBPAGE,
                        webpageUrl: message
                    }
                },
                scene: type
            }, function () {
                fnSuccess();
            }, function (reason) {
                fnFailure();
            });
        }

        $scope.showActionSheet = function () {

            if(SignService.userLogined()){
                if($scope.isMyFavourite){
                    btns = [
                        { text: '发送给朋友' },
                        { text: '分享到朋友圈' },
                        { text: '取消收藏' }
                    ];
                }else{
                    btns = [
                        { text: '发送给朋友' },
                        { text: '分享到朋友圈' },
                        { text: '收藏' }
                    ];
                }
            }else{
                btns = [
                    { text: '发送给朋友' },
                    { text: '分享到朋友圈' }
                ];
            }

            var address = "http://www.cntea168.com/teaApp/www/#/share/product/"+$stateParams.id;
            var name = "华夏茶叶交易中心-" +  $scope.productDetail.name;
            var thumb = $scope.productDetail.imagePrefix + $scope.productDetail.imageUrl +"@!wechat_thumb";
            $ionicActionSheet.show({
                buttons:btns,
                destructiveText: '',
                titleText: '',
                cancelText: '取消',
                cancel: function() {
                },
                buttonClicked: function(index) {
                    switch (index){
                        case 0:
                            $scope.shareViaWechat(address,"华夏茶叶交易中心",$scope.productDetail.name,'分享',Wechat.Scene.SESSION,thumb, function () {

                            }, function () {

                            });
                            break;
                        case 1:
                            $scope.shareViaWechat(address,name,"",'分享',Wechat.Scene.TIMELINE,thumb, function () {

                            }, function () {

                            });
                            break;
                        case 2:
                            FavouriteService.addFavourite(SignService.getUserId(),$stateParams.id).then(function (data) {
                                if(data.status){
                                    $scope.isMyFavourite = !$scope.isMyFavourite;
                                }
                            });
                            break
                    }
                    return true;
                }
            });
        }
        $scope.action = function () {
            $scope.showActionSheet();
        }
        $scope.show5Days = function () {
            $scope.loadChartData({id:$stateParams.id,interval:5,sampling:180})
        }

        $scope.showMonth = function () {
            $scope.loadChartData({id:$stateParams.id,interval:30,sampling:240})
        }

        $scope.showToday = function () {
            $scope.loadChartData($scope.f);
        }



    })
