
angular.module('OBApp.me', [
  'ngRoute',
  'ngTouch'
])

.config(function($stateProvider, $urlRouterProvider) {

  $stateProvider
  .state('tab.me', {
    url: '/me',
    cache:false,
    views: {
      'tab-me': {
        templateUrl: 'me/index.html',
        controller: 'MeController'
      }
    },
    resolve:{
      userInfo: function (SignService) {
        if(SignService.userLogined()){
          return SignService.getSignedUser({cache : false}).then(function(data){
            return data;
          });
        }else{
          return null;
        }
      }
    }
  })
  .state('tab.tradeList', {
    url: '/me/tradeList',
    views: {
      'tab-me': {
        templateUrl: 'me/tradeList.html',
        controller: 'MeTradeListController'
      }
    }
  })
  .state('tab.tradeDetail', {
    url: '/me/tradeDetail?tid',
    views: {
      'tab-me': {
        templateUrl: 'me/tradeDetail.html',
        controller: 'MeTradeDetailController'
      }
    }
  })
  .state('tab.ownProducts', {
    url: '/me/ownProducts',
    views: {
      'tab-me': {
        templateUrl: 'me/ownProductList.html',
        controller: 'MeOwnProductListController'
      }
    }
  })
  .state('tab.ownProductDetail', {
    url: '/me/ownProductDetail/{id}',
    views: {
      'tab-me': {
        templateUrl: 'me/ownProductDetail.html',
        controller: 'ProductDetailController'
      }
    }
  })
  .state('tab.ownFavourites', {
    url: '/me/ownFavourites',
    views: {
      'tab-me': {
        templateUrl: 'me/ownFavouritesList.html',
        controller: 'MeOwnFavouritesListController'
      }
    }
  })
  .state('tab.moneyLog', {
    url: '/me/moneyLog',
    views: {
      'tab-me': {
        templateUrl: 'me/moneyLog.html',
        controller: 'MoneyLogController'
      }
    }
  })
  .state('tab.profit', {
    url: '/me/profit',
    views: {
      'tab-me': {
        templateUrl: 'me/profit.html',
        controller: 'ProfitController'
      }
    }
  })

  .state('tab.charge', {
    url: '/me/charge',
    views: {
      'tab-me': {
        templateUrl: 'me/charge.html',
        controller: 'ChargeController'
      }
    }
  })

  .state('tab.withdrawal', {
    url: '/me/withdrawal',
    views: {
      'tab-me': {
        templateUrl: 'me/withdrawal.html',
        controller: 'WithdrawalController'
      }
    },
    resolve:{
      payInfo: function (SignService,TradeService) {
        return TradeService.queryUserPayInfo(SignService.getUserId(),"CARD").then(function(data){
          if(data.status){
              return data.data.isPerfect;
          }else{
              return data.status;
          }
        });
      }
    }
  })
  .state('tab.delivery', {
    url: '/me/delivery',
        cache:false,
    views: {
      'tab-me': {
        templateUrl: 'me/delivery.html',
        controller: 'DeliveryController'
      }
    }
  })
  .state('tab.deliveryDetail', {
    url: '/me/deliveryDetail?tid&amount&tname',
    views: {
      'tab-me': {
        templateUrl: 'me/deliveryDetail.html',
        controller: 'DeliveryDetailController'
      }
    },
    resolve:{
      payInfo: function (SignService,TradeService) {
        return TradeService.queryUserPayInfo(SignService.getUserId(),'ADDRESS').then(function(data){
          if(data.status){
            return data.data;
          }else{
            return null;
          }
        });
      }
    }
  })
  .state('tab.support', {
    url: '/me/support',
    views: {
      'tab-me': {
        templateUrl: 'news/detail.html',
        controller: 'MeSupportController'
      }
    }
  })
  .state('tab.promotion', {
    url: '/me/promotion',
    views: {
      'tab-me': {
        templateUrl: 'me/promotionList.html',
        controller: 'MePromotionController'
      }
    }
  })
  .state('tab.myInfo', {
    url: '/me/myInfo',
    views: {
      'tab-me': {
        templateUrl: 'me/myInfo.html',
        controller: 'MeInfoController'
      }
    }
  })
  .state('tab.myBankCard', {
    url: '/me/myBankCard',
    views: {
      'tab-me': {
        templateUrl: 'me/myCardInfo.html',
        controller: 'MeCardInfoController'
      }
    },
    resolve:{
      payInfo: function (SignService,TradeService) {
        return TradeService.queryUserPayInfo(SignService.getUserId(),'CARD').then(function(data){
          if(data.status){
            return data.data;
          }else{
            return null;
          }
        });
      }
    }
  })
  .state('tab.myAddressInfo', {
    url: '/me/myAddressInfo',
    views: {
      'tab-me': {
        templateUrl: 'me/myAddressInfo.html',
        controller: 'MeAddressInfoController'
      }
    },
    resolve:{
      payInfo: function (SignService,TradeService) {
        return TradeService.queryUserPayInfo(SignService.getUserId(),'ADDRESS').then(function(data){
          if(data.status){
            return data.data;
          }else{
            return null;
          }
        });
      }
    }
  })

  .state('tab.invite', {
    url: '/me/invite',
    views: {
      'tab-me': {
        templateUrl: 'me/invite.html',
        controller: 'MeInviterController'
      }
    }
  })
  .state('tab.order', {
    url: '/me/order',
    views: {
      'tab-me': {
        templateUrl: 'me/orderList.html',
        controller: 'MeOrdersController'
      }
    }
  })
})

.controller('DeliveryController', function ($scope, $http,TradeService,SignService,$state) {
      TradeService.getUserOwnProductList(SignService.getUserId()).then(function (data) {
        $scope.ownProductList = data;
      });
      $scope.selectProduct = function (item) {
        $state.go('tab.deliveryDetail',{tid:item.product.id,amount:item.amount,tname:item.product.name});
      }
})

.controller('DeliveryDetailController', function ($scope, $http,TradeService,SignService,payInfo,$stateParams,$ionicPopup,$state) {
      $scope.f = {};
      $scope.f.address = payInfo == null ? "" : payInfo.address;
      $scope.teaName = $stateParams.tname;
      $scope.f.allAmount = $stateParams.amount;
      $scope.f.teaId = $stateParams.tid;


      $scope.save = function () {
        $ionicPopup.confirm({
          title: "提交提货申请?",
          buttons: [{
            text: '取消',
            type: 'button-default button-small',
            onTap: function(e) {
            }
          }, {
            text: '确认',
            type: 'button-assertive button-small',
            onTap: function(e) {
              TradeService.saveDeliveryOrder(SignService.getUserId(),$scope.f.teaId,$scope.f.amount,$scope.f.address).then(function (data) {
                  if(data.status){
                    $state.go('tab.delivery');
                  }
              })
            }
          }]
        });

      }
})


.controller('MeController', function ($scope, $location, SignService, TradeService,userInfo,$state,$ionicPopup) {

  $scope.userLogined = function(){
    return SignService.userLogined();
  }
  if(SignService.userLogined()){
    TradeService.getUserCapatial().then(function(data){
      $scope.userCapatial= data;
    });
  }


  if($scope.userLogined()){
    $scope.userInfo = userInfo;
    $scope.userInfo.fullImageUrl = userInfo.imagePrefix + userInfo.imageUrl;
  }

  $scope.teaCharge = function () {
    //var url = "http://10.0.1.3:8888/api/pay/teaPay.html#/inputAmount?uid=" + SignService.getUserId();

    var url = "http://www.cntea168.com/api/pay/teaPay.html#/inputAmount?uid=" + SignService.getUserId();
    $scope.ref = cordova.InAppBrowser.open(url, '_system', 'location=yes,clearcache=yes,clearsessioncache=yes,hardwareback=no,closebuttoncaption=完成');
    $scope.ref.addEventListener('exit', function(event) {
      TradeService.getUserCapatial().then(function(data){
        $scope.userCapatial= data;
      });
      //$state.go($state.current, {}, {reload: true});
      $ionicPopup.alert({
        title: '支付完成',
        template: "支付结果请注意查收短信",
        buttons:[{ text: '确定' }]
      });
    });
  }

  $scope.featureList = [
    { id: 1 , icon:"me1", style:"assertive", title:"我的订单", url: "tab.order"},
    //{ id: 2 , icon:"ion-bag", style:"assertive", title:"我的委托", url: "tab.trade.order"},
    { id: 3 , icon:"me2", style:"balanced", title:"交易明细", url: "tab.tradeList"},
    //{ id: 4 , icon:"ion-briefcase", style:"calm", title:"我的持仓", url: "tab.ownProducts"},
    //{ id: 5 , icon:"ion-heart", style:"calm", title:"我的自选", url: "tab.ownFavourites"},
    { id: 6 , icon:"me3", style:"assertive", title:"资金明细", url: "tab.moneyLog"},
    { id: 7 , icon:"me4", style:"royal", title:"盈亏情况", url: "tab.profit"},
    { id: 8 , icon:"me5", style:"balanced", title:"分享给朋友", url: "tab.invite"},
    { id: 9 , icon:"me6", style:"positive", title:"我的推广", url: "tab.promotion"},
    { id: 10 , icon:"me7", style:"calm", title:"客户服务", url: "tab.support"},
    { id: 11 , icon:"me8", style:"positive", title:"我的银行卡", url: "tab.myBankCard"},
    { id: 12 , icon:"me9", style:"calm", title:"我的地址", url: "tab.myAddressInfo"}
  ];


  $scope.signOut = function(){
    SignService.signOut();
  }
})

.controller('MeTradeListController', function ($scope, $http,SignService,TradeService) {
      TradeService.queryTradeList(SignService.getUserId()).then(function (data) {
        $scope.tradeList = data;
      })
})

.controller('MeTradeDetailController', function ($scope, $http,SignService,TradeService,$stateParams,$filter) {

  TradeService.queryTradeList(SignService.getUserId()).then(function (data) {
    $scope.tradeList = data;
    $scope.item = $filter('filter')($scope.tradeList,{tradeId:$stateParams.tid},false)[0];
  })
})

.controller('MeOwnProductListController', function ($scope, $http,SignService,TradeService) {

  TradeService.getUserOwnProductList(SignService.getUserId()).then(function (data) {
    $scope.ownProductList = data;
  })


})
.controller('MeOwnFavouritesListController', function ($scope, $http,SignService,FavouriteService,$stateParams,$filter,$ionicPopup) {

  FavouriteService.getUserFavouriteProductList(SignService.getUserId()).then(function (data) {
    $scope.ownProductList = data;
  });

  $scope.trendMode = 'percent';
  $scope.trendTitle = function(){
    if($scope.orderBy === '-totalAmount'){
      return "成交量";
    }else{
      return $scope.trendMode === 'price' ? "涨幅(价格)" : "涨/跌(百分比)";
    }
  }

  $scope.action = function (tid) {
    var myPopup = $ionicPopup.show({

      template: '<label>取消关注?</label>',
      title: '提示',
      scope: $scope,
      buttons: [
        {
          text: '返回',
          type: 'button-default',
          onTap: function(e) {
            return -1;
          }
        },
        {
          text: '<b>取消关注</b>',
          type: 'button-energized',
          onTap: function(e) {
            return 0;
          }
        }
      ]
    });
    myPopup.then(function(index) {
      if(index != -1){
        FavouriteService.addFavourite(SignService.getUserId(),tid).then(function(data){
          if(data.status){
            FavouriteService.getUserFavouriteProductList(SignService.getUserId()).then(function (data) {
              $scope.ownProductList = data;
            });
          }
        });
      }
    });
  };
})



.controller('MoneyLogController', function ($scope, $http,TradeService) {

  TradeService.getMoneyLogs().then(function (data) {
    $scope.moneyLogs = data;
  })
})

.controller('ProfitController', function ($scope, $http,TradeService) {

  TradeService.getProductProfit().then(function (data) {
    $scope.productProfit = data;
  })
})

.controller('ChargeController', function ($scope, $location,TradeService, $stateParams,$ionicPopup) {
  $scope.f = { type : "CHARGE" };

  var showMessage = function (message) {
    return  $ionicPopup.show({

      template: '<label>'+ message +'</label>',
      title: '提示',
      scope: $scope,
      buttons: [
        {
          text: '返回',
          type: 'button-default',
          onTap: function(e) {
            return -1;
          }
        }
      ]
    });
  }
  $scope.save = function () {

    //$scope.ref = cordova.InAppBrowser.open('http://120.24.158.113:82/api/pay/teaPay.html#/inputAmount', '_blank', 'location=yes,clearcache=yes,clearsessioncache=yes,hardwareback=no');
    //$scope.ref.addEventListener('exit', function(event) {
    //  alert(event.url);
    //});

  }

  $scope.save1 = function(){
    TradeService.moneyAction($scope.f).then(function (data) {
      if(data.status){
        showMessage("充值成功").then(function (index) {
          $location.path("tab/me");
        })
      }else{
        showMessage("充值失败").then(function (index) {
        })
      }
    })
  }
})

.controller('WithdrawalController', function ($scope, $http,TradeService,payInfo,SignService,$location) {
  $scope.userInfoIsPerfect =  payInfo;
  $scope.buttonTitle = $scope.userInfoIsPerfect ? "提现" : "保存";
  $scope.f = {};

  $scope.save = function(){
    if($scope.userInfoIsPerfect){
      $scope.f.type = "WITHDRAWAL";
      TradeService.moneyAction($scope.f).then(function (data) {
        if(data.status){
          $location.path("tab/me");
        }
      })
    }else{
      TradeService.saveUserPayInfo(SignService.getUserId(),$scope.f.card_number,$scope.f.bank_name,'',$scope.f.realName,'CARD').then(function (data) {
        if(data.status){
          $scope.userInfoIsPerfect = true;
          $scope.buttonTitle = $scope.userInfoIsPerfect ? "提现" : "保存";
        }
      })
    }
  }
})
.controller('MeSupportController', function ($scope, $http,ContentService) {
      $scope.title = "客户服务";
      ContentService.queryContentsByType("SUPPORT").then(function (data) {
        $scope.newsItem = data[0];
      });
})
.controller('MePromotionController', function ($scope, $http,TradeService) {
      TradeService.queryPromotionList().then(function (data) {
    $scope.promotionList = data;
  });
})
.controller('MeInfoController', function ($scope,SignService) {
      $scope.userInfo = {};
      SignService.getSignedUser({cache:false}).then(function (data) {
        $scope.userInfo = data;
      })
})
.controller('MeCardInfoController', function ($scope,SignService,payInfo,TradeService,$location) {
      $scope.userInfo = payInfo;
      $scope.save = function () {
        TradeService.saveUserPayInfo(SignService.getUserId(),$scope.userInfo.cardNumber,$scope.userInfo.bankName,$scope.userInfo.address,$scope.userInfo.realName,'CARD').then(function (data) {
          if(data.status){
            $location.path("tab/me");
          }
        })
      }
})

.controller('MeAddressInfoController', function ($scope,SignService,payInfo,TradeService,$location) {
  $scope.userInfo = payInfo;
  $scope.save = function () {
    TradeService.saveUserPayInfo(SignService.getUserId(),$scope.userInfo.cardNumber,$scope.userInfo.bankName,$scope.userInfo.address,$scope.userInfo.realName,'ADDRESS').then(function (data) {
      if(data.status){
        $location.path("tab/me");
      }
    })
  }
})

.controller('MeOrdersController', function ($scope,obPayService) {
      obPayService.queryOrderByUser().then(function (data) {
        if(data.status){
          $scope.orders = data.data;
        }
      })
})

.controller('MeInviterController', function ($scope,SignService) {

      $scope.inviteViaWechat = function () {
        var inviterId = SignService.getUserId();
        var message = "http://www.cntea168.com/teaApp/www/#/user/signUp?inviter="+inviterId;
        Wechat.share({
          message: {
            title: "华夏茶叶交易中心",
            description: "关注茶叶金融化即是投资策略，也是爱好。茶文化传统产业发展必须进行创新，同互联网、金融结合，注入资本支持推动行业发展。",
            mediaTagName: "邀请链接",
            thumb: "http://tea.xzgapp.com/a6.jpg",
            media: {
              type: Wechat.Type.WEBPAGE,
              webpageUrl: message
            }
          },
          scene: Wechat.Scene.TIMELINE
        }, function () {
        }, function (reason) {
        });
      }
      $scope.inviteViaWechatSession = function () {
        var inviterId = SignService.getUserId();
        var message = "http://www.cntea168.com/teaApp/www/#/user/signUp?inviter="+inviterId;
        Wechat.share({
          message: {
            title: "华夏茶叶交易中心",
            description: "关注茶叶金融化即是投资策略，也是爱好。茶文化传统产业发展必须进行创新，同互联网、金融结合，注入资本支持推动行业发展。",
            mediaTagName: "邀请链接",
            thumb: "http://tea.xzgapp.com/a6.jpg",
            media: {
              type: Wechat.Type.WEBPAGE,
              webpageUrl: message
            }
          },
          scene: Wechat.Scene.SESSION
        }, function () {
        }, function (reason) {
        });
      }

})


